#import "MainImageView.h"

@implementation MainImageView

#pragma mark Initializations
- (id) init
{
	self = [super init];
	if (self != nil) {
		hueAdjust = 0.0;
		discBlur = 0.0;
	}
	return self;
}

-(void)awakeFromNib
{
	//[self loadImageFromPath:@"~/Desktop/starry+night+2+pretest2.jpg"];
	[hueValue setFloatValue:0.0];
	[discBlurValue setFloatValue:0.0];
	
}

#pragma mark -
#pragma mark Drawing Functions
-(void)drawRect:(NSRect)rect
{
	
	NSLog(@"redrawing now...");
	CIContext *context = [[NSGraphicsContext currentContext] CIContext];
    CGRect     cg = CGRectMake(NSMinX(rect), NSMinY(rect), NSWidth(rect), NSHeight(rect));
	if (mainImage == nil)
		NSLog(@"mainImage is nil!");
	
	[context drawImage:mainImage inRect:cg fromRect:cg];
	
}

- (IBAction)ApplyHue:(id)sender
{
	NSLog(@"applying hue adjust");
	CIImage *resultImage;
	CIFilter *hueAdjustFilter = [CIFilter filterWithName:@"CIHueAdjust"];
	[hueAdjustFilter setDefaults];
	[hueAdjustFilter setValue:mainImage forKey:@"inputImage"];
	[hueAdjustFilter setValue:[NSNumber numberWithFloat:[hueValue floatValue]] forKey:@"inputAngle"];
	resultImage = [hueAdjustFilter valueForKey:@"outputImage"];
	[mainImage release];
	mainImage = [resultImage retain];
	[self setNeedsDisplay:YES];
}

- (IBAction)applyDiscBlur:(id)sender
{
	NSLog(@"applying Disc Blur...");
	CIImage *resultImage;
	CIFilter *discBlurFilter = [CIFilter filterWithName:@"CIDiscBlur"];
	[discBlurFilter setDefaults];
	[discBlurFilter setValue:mainImage forKey:@"inputImage"];
	[discBlurFilter setValue:[NSNumber numberWithFloat:[discBlurValue floatValue]] forKey:@"inputRadius"];
	
	resultImage = [discBlurFilter valueForKey:@"outputImage"];
	mainImage = [resultImage retain];
	[self setNeedsDisplay:YES];
}

#pragma mark -
#pragma mark Image Loading/Saving
-(void)loadImageFromPath:(NSString *)aPath
{
	[imageLocation setStringValue:aPath];
	if(mainImage != nil)
		[mainImage release];
	
	NSURL *url = [NSURL fileURLWithPath:[aPath stringByStandardizingPath]];
	NSLog([url absoluteString]);
	//CIImage *currentImage = [CIImage imageWithContentsOfURL:url];
	mainImage = [[CIImage imageWithContentsOfURL:url]retain];
	if (mainImage == nil) {
		NSLog(@"failed to initialize the image");
	}
	
	CGRect mainImageSize = [mainImage extent];
	NSRect frame = [self bounds];
	[self setFrame:NSMakeRect(frame.origin.x, frame.origin.y, [mainImage extent].size.width, [mainImage extent].size.height)];
	[self setBoundsSize:NSMakeSize(mainImageSize.size.width, mainImageSize.size.height)];	
	[self setNeedsDisplay:YES];
	
}

- (IBAction)selectNewImage:(id)sender
{
	NSOpenPanel *openPanel = [NSOpenPanel openPanel];
	if ([openPanel runModal] == NSOKButton) {
		NSLog([openPanel filename]);
		NSString *fileLocation = [openPanel filename];
		[self loadImageFromPath:fileLocation];
	}
}

- (IBAction)saveImage:(id)sender
{
	NSData *data = [self dataWithPDFInsideRect:[self bounds]];
	[data writeToFile:[[NSString stringWithString:@"~/Desktop/test.pdf"]stringByStandardizingPath] atomically:NO];
}

#pragma mark -
#pragma mark Adjusting Sliders and Values...
- (IBAction)hueAdjustSliderDidChange:(id)sender
{
	hueAdjust = [sender floatValue];
	[self setNeedsDisplay:YES];
}

- (IBAction)discBlurSliderDidChange:(id)sender
{
	discBlur = [sender floatValue];
	[self setNeedsDisplay:YES];
}

- (void)resetAllValues
{

}

#pragma mark -
#pragma mark Notifications
-(void)windowWillClose:(NSNotification *)aNotification
{
	[NSApp terminate:self];
}


#pragma mark -
#pragma mark deallocations...
- (void) dealloc
{
	[mainImage release];
	[super dealloc];
}


@end
